#!/usr/bin/env python
# -*- coding: utf-8 -*-

__license__ = "BSD-3-Clause"
__author__ = "Marc Bourqui"
__version__ = "4.0.0"
__version_info__ = tuple(
    [
        int(num) if num.isdigit() else num
        for num in __version__.replace("-", ".", 1).split(".")
    ]
)
